	list		p=16F84
	#include	<p16f84.inc>

	__CONFIG	_WDT_OFF & _XT_OSC & _CP_OFF & _PWRTE_ON

;************************** DEKLARACIJE SPREMENLJIVK *********************

Temp1	equ	0x0C			
Temp2	equ	0x0D			; Zacasni spremenljivki

;*************************** DEFINICIJE NOVIH OZNAK **********************

	#define	RS	PORTB,1		; RS LCD linija
	#define	RW	PORTB,2		; RW LCD linija
	#define	E	PORTB,3		; E LCD linija

;****************** DEFINICIJE KONSTANT ZA LCD UKAZE *********************

Brisi	equ	0x01			; Brisanje LCD-ja
Home	equ	0x02			; Zaslon v originalno pozicijo
Kurzor0	equ	0x0C			; Izklop kurzorja
KurzorM	equ	0x0E			; Mali kurzor
KurzorV	equ	0x0F			; Velik kurzor, utripajo
DDRAM	equ	0x80			; OR maska za vpis v DDRAM
Desno	equ	0x1C			; Pomik zaslona v desno

;**************************** ZACETEK PROGRAMA ***************************

	org	0x000
	goto	Glavni			; Reset vektor

	org	0x004
Glavni					; Glavni program, prekinitev ni
	bsf	STATUS,RP0		; Banka 1
	clrf	TRISA
	clrf	TRISB			; Vsi pini izhodni
	movlw	b'10000001'
	movwf	OPTION_REG		; Timer povecuje notranja ura, preddelilnik
					; 1:4 (prekoracenje TMR0 v 1,024ms)
	bcf	STATUS,RP0		; Banka 0
	clrf	PORTA
	clrf	PORTB			; Vse pine postavimo na 0
	clrf	INTCON			; Onemogocimo vse prekinitve, T0IF je 0

	call	LCD_Init		; Inicializiramo LCD
					; trenutni DDRAM naslov je 0x00
	bsf	RS			; Podatkovni nacin, pisanje znakov
	movlw	'L'
	call	LCD_pisi
	movlw	'C'
	call	LCD_pisi
	movlw	'D'
	call	LCD_pisi		; Izpisemo besedo "LCD"
	movlw	0x40
	call	DDRAM_n			; Nastavimo naslov 2. vrstice
	bsf	RS			; Podatkovni nacin, pisanje
	movlw	'D'
	call	LCD_pisi
	movlw	'e'
	call	LCD_pisi	
	movlw	'l'
	call	LCD_pisi
	movlw	'a'
	call	LCD_pisi
	movlw	'!'
	call	LCD_pisi		; Izpisemo stavek "Dela!"
	call	Cakaj1S			; Cakamo 1s
	bcf	RS			; Ukazni nacin
	movlw	Kurzor0
	call	LCD_pisi		; Izklopimo kurzor
	call	Cakaj1S			; Cakamo 1s
	movlw	Desno
	call	LCD_pisi
	movlw	Desno
	call	LCD_pisi		; Pomaknemo zaslon za 2 znaka
					; v desno
	call	Cakaj1S			; Cakamo 1s
	movlw	Brisi
	call	LCD_pisi		; Izbrisemo DDRAM
	movlw	.2
	call	CakajMs			; Pocakamo 2ms
	movlw	Home
	call	LCD_pisi		; Zaslon v zacetno pozicijo
	movlw	.2
	call	CakajMs			; Pocakamo 2ms
	movlw	KurzorM
	call	LCD_pisi		; Vklopimo mali kurzor
	bsf	RS			; Podatkovni nacin
	movlw	'H'
	call	LCD_pisi
	movlw	'i'
	call	LCD_pisi
	movlw	's'
	call	LCD_pisi
	movlw	'a'
	call	LCD_pisi		; Izpisemo besedo "Hisa"
	call	Cakaj1S			; Pocakamo 1s
	bcf	RS			; Ukazni nacin
	movlw	0x00
	call	DDRAM_n			; Pomaknemo kurzor na crko 'H'
	bsf	RS			; Podatkovni nacin
	movlw	'N'
	call	LCD_pisi		; 'H' prepisemo z 'N'
	
Konec	goto	Konec			; Neskoncna zanka

;*************************** PODPROGRAMI *********************************

;********************* IZPISE VREDNOST IZ W NA LCD ***********************

LCD_pisi
	movwf	Temp1			; Shranimo W (podatek) v Temp1
	movlw	0x0F			; 0x0F -> W
	andwf	PORTB,f			; Izbrisemo zgornje 4 bite PORTB
	movf	Temp1,w			; W povrnemo nazaj iz Temp1
	andlw	0xF0			; Izbrisemo spodnje 4 bite W (podatka)
	iorwf	PORTB,f			; Posljemo zgornje 4 bite podatka
	bsf	E
	bcf	E			; Obvestimo LCD o novem podatku
	swapf	Temp1,f			; Zamenjamo nibble v Temp1
	movlw	0xF0			; 0xF0 -> W
	andwf	Temp1,f			; Izbrisemo spodnje 4 bite Temp1
	movlw	0x0F			; 0x0F -> W
	andwf	PORTB,f			; Izbrisemo zgornje 4 bite PORTB
	movf	Temp1,w			; Temp1 -> W
	iorwf	PORTB,f			; Posljemo spodnje 4 bite podatka
	bsf	E			; Potrdimo podatek
	bcf	E			; Obvestimo LCD o novem podatku
	call	Cakaj119u		; Pocakamo 119us
	return				; Konec podprograma

;****************** NAREDI ZAKASNITEV 119us ******************************

Cakaj119u
	movlw	.38			; Stevilo ponovitev zanke -> W
	movwf	Temp2			; W -> Temp2
Se	decfsz	Temp2,f			; Zmanjsaj Temp2 za 1
	goto	Se			; Ce Temp2 se ni 0, nadaljuj zanko
	return				; Temp0=0, konec podprograma

;****************** ZAKASNITEV PRIBLIZNO 1,03ms x W **********************

CakajMs
	movwf	Temp2			; Shrani stevilo ponovitev W -> Temp2
	clrf	TMR0			; Pobrise TMR0 register
Cak1	btfss	INTCON,T0IF		; Ali je timer ze pristel do konca?
	goto	Cak1			; Ne, preverjaj naprej
	bcf	INTCON,T0IF		; Da, izbrisi T0IF
	decfsz	Temp2,f			; Zmanjsaj stevec ponovitev
	goto	Cak1			; Stevec se ni 0, nadaljuj zanko
	return				; Stevec je 0, konec podprograma

;********************* INICIALIZIRA LCD **********************************

LCD_Init
	bcf	RW
	bcf	RS			; R/W in RS liniji na 0
	movlw	.15			; Parameter od CakajMs -> W
	call	CakajMs			; priblizno 15ms pavze za LCD
	movlw	0x30
	movwf	PORTB			; Na D7 - D4 posljemo 0x3
	bsf	E
	bcf	E			; Opozorimo LCD na nov podatek	
	movlw	.5
	call	CakajMs			; Priblizno 5ms pavze
	movlw	0x30
	movwf	PORTB			; Na D7 - D4 posljemo 0x3
	bsf	E
	bcf	E			; Opozorimo LCD na nov podatek
	call	Cakaj119u		; 119us pavze
	movlw	0x30
	movwf	PORTB			; Na D7 - D4 posljemo 0x3
	bsf	E
	bcf	E			; Opozorimo LCD na nov podatek
	movlw	.5
	call	CakajMs			; Priblizno 5ms pavze
	movlw	0x20
	movwf	PORTB			; Na D7 - D4 posljemo 0x2
	bsf	E
	bcf	E			; Opozorimo LCD na nov podatek
	call 	Cakaj119u		; 119us pavze
					; Sedaj smo v 4-bitnem nacinu
	movlw	0x28			; 2x16 znakov, 5x8 znaki, 4-bitno
	call	LCD_pisi		; Posljemo 0x28 na LCD
	movlw	0x08			; Izklop zaslona
	call	LCD_pisi		; Posljemo 0x08 na LCD
	movlw	0x01			; Brisanje DDRAM-a
	call	LCD_pisi		; Posljemo 0x01, izbrisemo zaslon
	movlw	.2
	call	CakajMs			; Zahtevana zakasnitev >2ms
	movlw	0x06			; pisanje v desno, brez pomikanja zaslona
	call	LCD_pisi		; Posljemo 0x06 na LCD
	movlw	KurzorV			; vklp zaslona, velik kurzor
	call	LCD_pisi		; Posljemo 0x0F na LCD

	return				; Konec podprograma

;********************* ZAKASNITEV PRIBLIZNO 1s ***************************
Cakaj1S	
	movlw	.250			; 250ms -> W
	call	CakajMs
	call	CakajMs
	call	CakajMs
	call	CakajMs			; 4 x 250ms = 1s (W se ohranja)
	return

;******************** NASTAVITEV TRENUTNEGA NASLOVA DDRAM-a **************
;******************** NASLOV VPISEMO V W PRED KLICEM PODPROGRAMA! ********
DDRAM_n
	bcf	RS			; izklopimo RS
	iorlw	DDRAM			; Z OR operacijo ustvarimo ukaz
	call	LCD_pisi		; Vpis ukaza v LCD
	return

	END

