        list        p=16f84
        #include    <p16f84.inc>
    
        __CONFIG    _WDT_OFF & _PWRTE_ON & _XT_OSC & _CP_OFF
     
        CBlock  0x0C
            W_reg, S_reg;       ; Zacasni spremenljivki za W in
                                ; STATUS v prekinitveni rutini
            RX_reg,TX_reg;      ; Sprejemni in oddajni register
            Stevec, Stevec2;    ; Stevca v programu
            Svez_P;             ; Indikator svezega podatka na
                                ; RS232 liniji
            Cas;                ; Pomozna spremenljivka za zakasnitev
            RAM_niz;             ; Naslov zacetka niza v RAM-u
        ENDC
        
Per     EQU     .31             ; Vrednost TMR0 za RS232 periodo
PolPer  EQU     .13             ; Vrednost TMR0 za pol periode RS232

; Per=31, PolPer=13 --> 9600 bit/s
; Per=13, PolPer=4  --> 19200 bit/s

CR      EQU     .13             ; Vrednost Carriage Return ASCII kode
LF      EQU     .10             ; Vrednost Line Feed ASCII kode
        
        org 0x000               ; Reset vektor
        goto    Glavni          ; Skok v glavni program
        
        org 0x004               ; Prekinitveni vektor
        
Prek                            ; Prekinitvena rutina
        bcf     INTCON,GIE      ; Izklopimo vse prekinitve
        movwf   W_reg           ; W -> W_reg
        swapf   STATUS,w        
        movwf   S_reg           ; STATUS -> S_reg (SWAPF!)
        btfss   INTCON,INTF     ; Ali imamo nov podatek na RS232?
        goto    Ni_podatka      ; Ne, ni novega podatka
        movlw   .8
        movwf   Stevec          ; 8 -> Stevec (stevilo sprejetih bitov)
        movlw   PolPer
        call    Cakaj           ; Napravimo zakasnitev do sredine START bita
Delaj
        movlw   Per
        call    Cakaj           ; Zakasnitev do LSB podatka
        bcf     STATUS,C        ; C=0
        btfsc   PORTB,0         ; Ali je podatek enak 1?
        bsf     STATUS,C        ; Da, torej C=1
        rrf     RX_reg,f        ; Shranimo dobljeni bit v RX_reg
        decfsz  Stevec,f        ; Ali je stevec ze 0?
        goto    Delaj           ; Ne, sprejemaj naprej
        call    Cakaj           ; Da
        movlw   PolPer
        call    Cakaj           ; Sinhroniziramo PIC polno periodo               
        bcf     INTCON,INTF     ; Pobrisemo INTF
        bsf     Svez_P,0        ; Oznacimo svez podatek
Ni_podatka
        swapf   S_reg,w
        movwf   STATUS          ; S_reg -> STATUS
        movf    W_reg,w         ; W_reg -> W
        bsf     INTCON,GIE      ; Omogocimo vse prekinitve
        retfie                  ; Izhod iz prekinitvene rutine

;*************************** GLAVNI PROGRAM ******************************

Glavni
        bcf     STATUS,RP0      ; Banka 0
        clrf    Svez_P          ; Inicializiramo indikator svezega podatka
        movlw   b'10010000'
        movwf   INTCON          ; GIE=1, INTE=1
        bsf     STATUS,RP0      ; Banka 1
        movlw   b'10000000'
        movwf   OPTION_REG      ; Prekinitev ob padanju signala na RB0
        movlw   b'00000001'     ; RB0 je vhod
        movwf   TRISB           ; RB0 je vhod, ostali RBx so izhodi
        bcf     STATUS,RP0      ; Banka 0
        bsf     PORTB,1         ; Zacetno stanje RS232 linije (1)
        call    RAM_init        ; V RAM vpise besedo "16F84"
         
Krozi   
        btfss   Svez_P,0        ; Ali je podatek svez?
        goto    Krozi           ; Ne, preverjaj naprej
        clrf    Svez_P          ; Da, izbrisi indikator
        movlw   'R'
        subwf   RX_reg,W        ; Testiramo, ali je sprejeti podatek crka R
        btfsc   STATUS,Z        ; Ali je sprejeta crka R?
        call    RAM_na_RS232    ; Da, izpisi besedo iz RAM-a
        call    RS232_stavek    ; Izpisi stavek
        movf    RX_reg,W        ; RX_reg -> W
        movwf   TX_reg          ; W -> TX_reg
        call    Poslji_RS232    ; Posljemo sprejeti podatek nazaj
        movlw   CR
        movwf   TX_reg
        call    Poslji_RS232
        movlw   LF
        movwf   TX_reg
        call    Poslji_RS232    ; Posljemo CR in LF (prehod v novo 
                                ; vrstico v terminalskem programu)
        goto    Krozi           ; Neskoncna zanka

;**************************** PODPROGRAMI ********************************

;*** PODPROGAM ZA ZAKASNITEV, VREDNOST ZAKASNITVE PODAMO V REGISTRU W ****

Cakaj                           ; Podprogram, ki naredi zakasnitev 1 periode
        movwf   Cas             ; W -> Cas
Ni_se
        decfsz  Cas,f           ; Ali je ze pretekel cas?
        goto    Ni_se           ; Ne se, cakaj naprej
        return                  ; Da, koncaj
        
;******** PODPROGRAM, KI NA RS232 POSLJE PODATEK IZ REGISTRA TX_reg ******

Poslji_RS232
        bcf     INTCON,GIE      ; Izklopimo vse prekinitve (torej onemogocimo
                                ; sprejem iz RS232 linije in ostale motnje)
        movlw   .8
        movwf   Stevec          ; 8 ponovitev -> Stevec
        movlw   Per
        bcf     PORTB,1         ; Zacni START bit
Poslji
        call    Cakaj           ; Napravi 1 bit pavze
        rrf     TX_reg,f        ; LSB -> carry
        btfsc   STATUS,C        ; Ali je LSB bit enak 1?
        goto    Vklopi          ; Da, vklopi RS232 linijo
        bcf     PORTB,1         ; Ne, izklopi RS232 linijo
        goto    Konec
Vklopi  bsf     PORTB,1         ; Vklopi RS232 linijo
Konec   decfsz  Stevec,f        ; So ze vsi podatki preneseni?
        goto    Poslji          ; Ne, delaj naprej
        call    Cakaj           ; Da, poslji se zadnji bit
        bsf     PORTB,1         ; Poslji STOP bit
        call    Cakaj
        bcf     INTCON,INTF     ; Onemogocimo nepopoln sprejem z RS232
                                ; linije
        bsf     INTCON,GIE      ; Zopet omogocimo prekinitve (sprejem)
        return                  ; Koncaj podprogram
        
;******************** TABELA S SPOROCILOM ********************************

Stavek  addwf   PCL,f           ; Tabela s sporocilom
        DT      "Poslal si mi crko "
        
;********* PODPROGRAM, KI NA RS232 IZPISE STAVEK IZ TABELE Stavek ********

RS232_stavek
        clrf   Stevec2          ; Inicializacija stevca
Izpisuj
        movf    Stevec2,W       ; Stevec2 -> W
        call    Stavek          ; Pogledamo naslednjo crko v stavku
        movwf   TX_reg          ; Crka -> TX_reg (oddajni register)
        call    Poslji_RS232    ; Posljemo crko na RS232 linijo
        incf    Stevec2,f       ; Naslednja crka
        movlw   .18
        subwf   Stevec2,W
        btfss   STATUS,Z        ; Ali je izpisanih vseh 18 crk?
        goto    Izpisuj         ; Ne, nadaljuj izpisovanje
        return                  ; Poslan je bil cel niz
        
;********************* VPISE BESEDO "16F84" V NIZ V RAM-u ****************
RAM_init
        movlw   '1'
        movwf   RAM_niz
        movlw   '6'
        movwf   RAM_niz+1
        movlw   'F'
        movwf   RAM_niz+2
        movlw   '8'
        movwf   RAM_niz+3
        movlw   '4'
        movwf   RAM_niz+4
        movlw   CR
        movwf   RAM_niz+5
        movlw   LF
        movwf   RAM_niz+6       ; Se skok v novo vrstico...
        return                  ; Beseda je sedaj v RAM-u, izhod
        
;** PODPROGRAM ZA POSILJANJE BESEDE IZ RAM-a Z INDIREKTNIM NASLAVLJANJEM *

RAM_na_RS232
        movlw   RAM_niz
        movwf   FSR             ; V FSR vpisemo naslov zelenega podatka
                                ; v RAM-u
Nov_P
        movf    INDF,W          ; Iskani podatek damo v W...
        movwf   TX_reg          ; ...in nato v TX_reg...
        call    Poslji_RS232    ; ...ter na RS232 linijo.
        incf    FSR,f           ; Nastavimo naslednji naslov
        movlw   RAM_niz+7
        subwf   FSR,W
        btfss   STATUS,Z        ; Ali smo izpisali vse znake?
        goto    Nov_P           ; Ne se, nadaljujmo
        return                  ; Da, zapustimo podprogram

                      
        END                     ; Konec programa
