	list		p=16F84		; Tip mikrokontrolerja
	#include	<p16f84.inc>	; Vljucitev .inc datoteke

	__CONFIG	_WDT_OFF & _XT_OSC & _CP_OFF & _PWRTE_ON

;************************** DEKLARACIJE SPREMENLJIVK *********************

	CBlock	0x0C		; Blok konstant
		Temp1
		Temp2		; Zacasni spremenljivki	
		Stevec : 2	; 16-bitni stevec
		Cifra		; Cifra za izpis
		Nicla		; Indikator nicle
		Podatek : 2	; 16-bitni podatek za izpis na LCD
		Deljitelj : 2	; 16-bitni deljitelj za pretvorbo
	endc			; Konec bloka konstant

;*************************** DEFINICIJE NOVIH OZNAK **********************

	#define	RS	PORTB,1		; RS LCD linija
	#define	RW	PORTB,2		; RW LCD linija
	#define	E	PORTB,3		; E LCD linija

;****************** DEFINICIJE KONSTANT ZA LCD ***************************

DDRAM	equ	0x80			; OR maska za vpis v DDRAM

;**************************** ZACETEK PROGRAMA ***************************

	org	0x000
	goto	Glavni			; Reset vektor

	org	0x004
Glavni					; Glavni program, prekinitev ni
	bsf	STATUS,RP0		; Banka 1
	clrf	TRISA
	clrf	TRISB			; Vsi pini izhodni
	movlw	b'10000001'
	movwf	OPTION_REG		; Timer povecuje notranja ura, preddelilnik
					; 1:4 (prekoracenje TMR0 v 1,024ms)
	bcf	STATUS,RP0		; Banka 0
	clrf	PORTA
	clrf	PORTB			; Vse pine postavimo na 0
	clrf	INTCON			; Onemogocimo vse prekinitve, T0IF je 0
	call	LCD_Init		; Inicializiramo LCD
					; trenutni DDRAM naslov je 0x00
	clrf	Stevec
	clrf	Stevec+1		; Izbrisemo Stevec
Izvajaj
	bcf	RS			; Ukazni nacin
	movlw	DDRAM
	call	LCD_pisi		; Pisi na zacetek vrstice
	bsf	RS			; Podatkovni nacin
	movf	Stevec,w
	movwf	Podatek
	movf	Stevec+1,w
	movwf	Podatek+1		; Stevec -> Podatek
	call	Pisi_st			; Izpisemo Stevilo na LCD
	incf	Stevec+1,f
	btfsc	STATUS,Z
	incf	Stevec,f		; Povecamo 16-bitni Stevec za 1
	movlw	.250			; 250ms -> W
	call	CakajMs			; 250ms zakasnitve
	goto	Izvajaj			; Neskoncna zanka

;*************************** PODPROGRAMI *********************************

;********************* IZPISE VREDNOST IZ W NA LCD ***********************

LCD_pisi
	movwf	Temp1			; Shranimo W (podatek) v Temp1
	movlw	0x0F			; 0x0F -> W
	andwf	PORTB,f			; Izbrisemo zgornje 4 bite PORTB
	movf	Temp1,w			; W povrnemo nazaj iz Temp1
	andlw	0xF0			; Izbrisemo spodnje 4 bite W (podatka)
	iorwf	PORTB,f			; Posljemo zgornje 4 bite podatka
	bsf	E
	bcf	E			; Obvestimo LCD o novem podatku
	swapf	Temp1,f			; Zamenjamo nibble v Temp1
	movlw	0xF0			; 0xF0 -> W
	andwf	Temp1,f			; Izbrisemo spodnje 4 bite Temp1
	movlw	0x0F			; 0x0F -> W
	andwf	PORTB,f			; Izbrisemo zgornje 4 bite PORTB
	movf	Temp1,w			; Temp1 -> W
	iorwf	PORTB,f			; Posljemo spodnje 4 bite podatka
	bsf	E
	bcf	E			; Obvestimo LCD o novem podatku
	call	Cakaj119u		; Pocakamo 119us
	return				; Konec podprograma

;****************** NAREDI ZAKASNITEV 119us ******************************

Cakaj119u
	movlw	.38			; Stevilo ponovitev zanke -> W
	movwf	Temp2			; W -> Temp2
Se	decfsz	Temp2,f			; Zmanjsaj Temp2 za 1
	goto	Se			; Ce Temp2 se ni 0, nadaljuj zanko
	return				; Temp0=0, konec podprograma

;****************** ZAKASNITEV PRIBLIZNO 1,03ms x W **********************

CakajMs
	movwf	Temp2			; Shrani stevilo ponovitev W -> Temp2
	clrf	TMR0			; Pobrise TMR0 register
Cak1	btfss	INTCON,T0IF		; Ali je timer ze pristel do konca?
	goto	Cak1			; Ne, preverjaj naprej
	bcf	INTCON,T0IF		; Da, izbrisi T0IF
	decfsz	Temp2,f			; Zmanjsaj stevec ponovitev
	goto	Cak1			; Stevec se ni 0, nadaljuj zanko
	return				; Stevec je 0, konec podprograma

;********************* INICIALIZIRA LCD **********************************

LCD_Init
	bcf	RW
	bcf	RS			; R/W in RS liniji na 0
	movlw	.15			; Parameter od CakajMs -> W
	call	CakajMs			; priblizno 15ms pavze za LCD
	movlw	0x30
	movwf	PORTB			; Na D7 - D4 posljemo 0x3
	bsf	E
	bcf	E			; Opozorimo LCD na nov podatek	
	movlw	.5
	call	CakajMs			; Priblizno 5ms pavze
	movlw	0x30
	movwf	PORTB			; Na D7 - D4 posljemo 0x3
	bsf	E
	bcf	E			; Opozorimo LCD na nov podatek
	call	Cakaj119u		; 119us pavze
	movlw	0x30
	movwf	PORTB			; Na D7 - D4 posljemo 0x3
	bsf	E
	bcf	E			; Opozorimo LCD na nov podatek
	movlw	.5
	call	CakajMs			; Priblizno 5ms pavze
	movlw	0x20
	movwf	PORTB			; Na D7 - D4 posljemo 0x2
	bsf	E
	bcf	E			; Opozorimo LCD na nov podatek
	call 	Cakaj119u		; 119us pavze
					; Sedaj smo v 4-bitnem nacinu
	movlw	0x28			; 2x16 znakov, 5x8 znaki, 4-bitno
	call	LCD_pisi		; Posljemo 0x28 na LCD
	movlw	0x08			; Izklop zaslona
	call	LCD_pisi		; Posljemo 0x08 na LCD
	movlw	0x01			; Brisanje DDRAM-a
	call	LCD_pisi		; Posljemo 0x01, izbrisemo zaslon
	movlw	.2
	call	CakajMs			; Zahtevana zakasnitev >2ms
	movlw	0x06			; pisanje v desno, brez pomikanja zaslona
	call	LCD_pisi		; Posljemo 0x06 na LCD
	movlw	0x0F			; vklop zaslona, velik kurzor
	call	LCD_pisi		; Posljemo 0x0F na LCD
	return				; Konec podprograma

;************** PODPROGRAM ZA 16-BITNO PRETVARJANJE **********************

Pretvori16				; 16-bitno pretvarjanje
	clrf	Cifra			; Izbrisemo cifro
Nadaljuj
	movf	Deljitelj+1,w		; Nizji bajt -> W
	subwf	Podatek+1,f		; Odstej nizje bajte
	movf	Deljitelj,w		; Nalozi Deljitelj -> W
	btfss	STATUS,C		; Ali je prislo do prenosa?
	incf	Deljitelj,w		; Da, napravi prenos
	subwf	Podatek,f		; Odstej visje bajte
	btfsc	STATUS,C		; Ali je rezultat pozitiven?
	goto	Ena_vec			; Da, skoci na Ena_vec
	movf	Deljitelj+1,w		; Ne
	addwf	Podatek+1,f		; Naredi rezultat zopet pozitiven
	btfsc	STATUS,C		; pristeje, kar je prej odstel
	incf	Podatek,f		; Upostevanje prenosa
	movf	Deljitelj,w
	addwf	Podatek,f		; Izracun ostanka po "deljenju"
	movf	Cifra,w			; Cifra -> W
	addlw	'0'			; Izracunaj ASCII kodo
	btfsc	Nicla,0			; Ali je to vodilna nicla?
	movlw	' '			; Da, izpisi presledek
	call	LCD_pisi		; Ne, izpisi cifro na LCD
	return				; Izhod iz podprograma
Ena_vec
	bcf	Nicla,0			; Izklopimo indikator nicle
	incf	Cifra,f			; Povecamo cifro
	goto	Nadaljuj		; Nadaljuj pretvorbo
	
;**************	PODPROGRAM ZA IZPIS STEVILA IZ STEV NA LCD ***************

Pisi_st	
	bsf	Nicla,0			; Vklopimo indikator nicle

	movlw	0x27
	movwf	Deljitelj
	movlw	0x10
	movwf	Deljitelj+1		; 10000 -> Deljitelj
	call	Pretvori16		; Pretvorimo desettisocice

	movlw	0x03
	movwf	Deljitelj
	movlw	0xE8
	movwf	Deljitelj+1		; 1000 -> Deljitelj
	call	Pretvori16		; Pretvorimo tisocice

	clrf	Deljitelj
	movlw	.100			; 100 -> Deljitelj
	movwf	Deljitelj+1		; Pretvorimo stotice
	call	Pretvori16
	
	movlw	.10			; 10 -> Deljitelj
	movwf	Deljitelj+1		; Pretvorimo desetice
	call	Pretvori16

	movf	Podatek+1,w		; Enice -> W
	addlw	'0'			; Pretvorba v ASCII kodo
	call	LCD_pisi		; Izpis enic na LCD
	return				; Konec podprograma
	
	END				; Konec programa
